import { Identifiable, Identifier, InternshipProgramEntry } from "@/data";
import { CourseDTO, courseDtoTransformer } from "@/api/dto/course";
import { OneWayTransformer, Transformer } from "@/serialization";
import { Edition } from "@/data/edition";
import moment from "moment-timezone";
import { Subset } from "@/helpers";
import { InternshipTypeDTO, internshipTypeDtoTransformer } from "@/api/dto/type";
import { ReportFieldDefinition, ReportFieldType } from "@/data/report";

export interface ProgramEntryDTO extends Identifiable {
    description: string;
    descriptionEng: string;
}

export interface EditionDTO extends Identifiable {
    editionStart: string,
    editionFinish: string,
    reportingStart: string,
    course: CourseDTO,
    availableSubjects: ProgramEntryDTO[],
    availableInternshipTypes: InternshipTypeDTO[],
    reportSchema: FieldDefinitionDTO[],
}

export interface EditionTeaserDTO extends Identifiable {
    editionStart: string,
    editionFinish: string,
    courseName: string,
}

export enum FieldDefinitionDTOType {
    LongText = "LongText",
    ShortText = "ShortText",
    Select = "Select",
    Radial = "Radial",
    Checkbox = "Checkbox",
}

export const fieldDefinitionDtoTypeTransformer: Transformer<FieldDefinitionDTOType, ReportFieldType> = {
    transform(dto: FieldDefinitionDTOType, context?: unknown) {
        switch (dto) {
            case FieldDefinitionDTOType.LongText:
                return "long-text"
            case FieldDefinitionDTOType.ShortText:
                return "short-text";
            case FieldDefinitionDTOType.Select:
                return "select";
            case FieldDefinitionDTOType.Radial:
                return "radio";
            case FieldDefinitionDTOType.Checkbox:
                return "checkbox";
        }
    },
    reverseTransform(type: ReportFieldType, context?: unknown) {
        switch (type) {
            case "short-text":
                return FieldDefinitionDTOType.ShortText;
            case "long-text":
                return FieldDefinitionDTOType.LongText;
            case "checkbox":
                return FieldDefinitionDTOType.Checkbox;
            case "radio":
                return FieldDefinitionDTOType.Radial;
            case "select":
                return FieldDefinitionDTOType.Select;
        }
    }
}

export interface FieldDefinitionDTO extends Identifiable {
    label: string;
    labelEng: string;
    description: string;
    descriptionEng: string;
    fieldType: FieldDefinitionDTOType;
    choices: string[];
}

export const fieldDefinitionDtoTransformer: Transformer<FieldDefinitionDTO, ReportFieldDefinition> = {
    transform(dto: FieldDefinitionDTO, context?: unknown): ReportFieldDefinition {
        return {
            id: dto.id,
            choices: (dto.choices || []).map(choice => JSON.parse(choice)),
            description: {
                pl: dto.description,
                en: dto.descriptionEng,
            },
            label: {
                pl: dto.label,
                en: dto.labelEng,
            },
            type: fieldDefinitionDtoTypeTransformer.transform(dto.fieldType),
        }
    },
    reverseTransform(subject: ReportFieldDefinition, context?: unknown): FieldDefinitionDTO {
        return {
            id: subject.id,
            choices: "choices" in subject && subject.choices.map(choice => JSON.stringify(choice)) || [],
            description: subject.description.pl,
            descriptionEng: subject.description.en,
            fieldType: fieldDefinitionDtoTypeTransformer.reverseTransform(subject.type),
            label: subject.label.pl,
            labelEng: subject.label.en,
        }
    }
}

export const editionTeaserDtoTransformer: OneWayTransformer<EditionTeaserDTO, Subset<Edition>> = {
    transform(subject: EditionTeaserDTO, context?: undefined): Subset<Edition> {
        return subject && {
            id: subject.id,
            startDate: moment(subject.editionStart),
            endDate: moment(subject.editionFinish),
            course: {
                name: subject.courseName,
            }
        }
    }
}

export const editionDtoTransformer: Transformer<EditionDTO, Edition> = {
    reverseTransform(subject: Edition, context: undefined): EditionDTO {
        return {
            id: subject.id,
            editionFinish: subject.endDate.toISOString(),
            editionStart: subject.startDate.toISOString(),
            course: courseDtoTransformer.reverseTransform(subject.course),
            reportingStart: subject.reportingStart.toISOString(),
            availableSubjects: subject.program.map(entry => programEntryDtoTransformer.reverseTransform(entry)),
            availableInternshipTypes: subject.types.map(entry => internshipTypeDtoTransformer.reverseTransform(entry)),
            reportSchema: subject.schema.map(entry => fieldDefinitionDtoTransformer.reverseTransform(entry)),
        };
    },
    transform(subject: EditionDTO, context: undefined): Edition {
        return {
            id: subject.id,
            course: courseDtoTransformer.transform(subject.course),
            startDate: moment(subject.editionStart),
            endDate: moment(subject.editionFinish),
            minimumInternshipHours: 40,
            maximumInternshipHours: 160,
            proposalDeadline: moment(subject.reportingStart),
            reportingStart: moment(subject.reportingStart),
            reportingEnd: moment(subject.reportingStart).add(1, 'month'),
            program: (subject.availableSubjects || []).map(entry => programEntryDtoTransformer.transform(entry)),
            types: (subject.availableInternshipTypes || []).map(entry => internshipTypeDtoTransformer.transform(entry)),
            schema: (subject.reportSchema || []).map(entry => fieldDefinitionDtoTransformer.transform(entry)),
        };
    }
}

export const programEntryDtoTransformer: Transformer<ProgramEntryDTO, InternshipProgramEntry> = {
    transform(subject: ProgramEntryDTO, context: never): InternshipProgramEntry {
        return {
            id: subject.id,
            description: subject.description,
        }
    },
    reverseTransform(subject: InternshipProgramEntry, context: never): ProgramEntryDTO {
        return {
            id: subject.id,
            description: subject.description,
            descriptionEng: "",
        }
    },
}

interface EditionUpdateDTO extends Identifiable {
    editionStart: string;
    editionFinish: string;
    reportingStart: string;
    course: CourseDTO;
    availableSubjectsIds: Identifier[],
    availableInternshipTypesIds: Identifier[],
    reportSchema: Identifier[],
}

export const editionUpdateDtoTransformer: OneWayTransformer<Edition, EditionUpdateDTO> = {
    transform(subject: Edition, context?: undefined): EditionUpdateDTO {
        return {
            id: subject.id,
            editionFinish: subject.endDate.toISOString(),
            editionStart: subject.startDate.toISOString(),
            course: courseDtoTransformer.reverseTransform(subject.course),
            reportingStart: subject.reportingStart.toISOString(),
            availableSubjectsIds: subject.program.map(x => x.id).filter(x => typeof x !== "undefined") as Identifier[],
            availableInternshipTypesIds: subject.types.map(x => x.id).filter(x => typeof x !== "undefined") as Identifier[],
            reportSchema: subject.schema.map(x => x.id).filter(x => typeof x !== "undefined") as Identifier[],
        }
    }
}
