import React, { useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useAsyncState } from "@/hooks";
import { useSpacing } from "@/styles";
import api from "@/management/api";
import { Box, Button, Container, Dialog, DialogActions, DialogContent, DialogTitle, IconButton, Tooltip, Typography } from "@material-ui/core";
import MaterialTable, { Column } from "material-table";
import { actionsColumn } from "@/management/common/helpers";
import {
    Account,
    BriefcaseAccount,
    BriefcaseAccountOutline, CertificateOutline,
    FileChartOutline,
    FileFind,
    FormatPageBreak,
    Refresh,
    Star,
    StickerCheckOutline
} from "mdi-material-ui";
import { Page } from "@/pages/base";
import { Actions } from "@/components";
import { BulkActions } from "@/management/common/BulkActions";
import { Async } from "@/components/async";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { EditionManagement, EditionManagementProps } from "@/management/edition/manage";
import { InternshipSubmission } from "@/management/api/internship";
import { createPortal } from "react-dom";
import { FileInfo } from "@/components/fileinfo";
import { StepState } from "@/management/edition/common/StepState";
import { fullname, Internship, isStudentDataComplete, Student } from "@/data";
import { GradeDialog } from "@/management/edition/internship/grade";
import { InternshipDetailsDialog } from "@/management/edition/proposal/details";
import { AcceptanceActions } from "@/components/acceptance-action";
import { InternshipDocument } from "@/api/dto/internship-registration";
import { StudentPreview } from "@/pages/user/profile";
import { SubmissionStatus } from "@/state/reducer/submission";

const title = "edition.internships.title";

export const canGrade = (internship: InternshipSubmission) => !!(internship);

const ProposalAction = ({ internship, children }: { internship: InternshipSubmission, children: React.ReactChild }) => {
    const { t } = useTranslation();
    const [open, setOpen] = useState(false);

    const handleDiscard = (comment: string) => {
        setOpen(false);
        api.internship.discard(internship as Internship, comment);
    }

    const handleAccept = (comment?: string) => {
        setOpen(false);
        api.internship.accept(internship as Internship, comment);
    }

    return <>
        <div onClick={ () => setOpen(true) } style={{ display: "inline-block", cursor: "pointer" }}>{ children || <FileFind /> }</div>
        { createPortal(
            <InternshipDetailsDialog onDiscard={ handleDiscard } onAccept={ handleAccept } internship={ internship } open={ open } onClose={ () => setOpen(false) }/>,
            document.getElementById("modals") as Element,
        ) }
    </>;
}

const IPPAction = ({ internship, children }: { internship: InternshipSubmission, children: React.ReactChild }) => {
    const { t } = useTranslation();
    const [open, setOpen] = useState(false);

    const handleDiscard = (comment: string) => {
        setOpen(false);
        api.document.discard(internship.ipp as InternshipDocument, comment);
    }

    const handleAccept = (comment?: string) => {
        setOpen(false);
        api.document.accept(internship.ipp as InternshipDocument, comment);
    }

    return <>
        { internship.ipp
            ? <div onClick={ () => setOpen(true) } style={{ display: "inline-block", cursor: "pointer" }}>{ children }</div>
            : children }
        { createPortal(
            <Dialog maxWidth="md" fullWidth open={ open } onClose={ () => setOpen(false) }>
                <DialogTitle>{ fullname(internship.intern as Student) }</DialogTitle>
                <DialogContent>
                    { internship.ipp && <FileInfo document={ internship.ipp } /> }
                </DialogContent>
                <DialogActions>
                    <AcceptanceActions onAccept={ handleAccept } onDiscard={ handleDiscard } label="internship"/>
                </DialogActions>
            </Dialog>,
            document.getElementById("modals") as Element,
        ) }
    </>;
}

const StudentAction = ({ internship, children }: { internship: InternshipSubmission, children: React.ReactChild }) => {
    const { t } = useTranslation();
    const [open, setOpen] = useState(false);

    return <>
        <div onClick={ () => setOpen(true) } style={{ display: "inline-block", cursor: "pointer" }}>{ children }</div>
        { createPortal(
            <Dialog maxWidth="md" fullWidth open={ open } onClose={ () => setOpen(false) }>
                <DialogTitle>{ fullname(internship.intern as Student) }</DialogTitle>
                <DialogContent>
                    { internship.intern && <StudentPreview student={ internship.intern } /> }
                </DialogContent>
            </Dialog>,
            document.getElementById("modals") as Element,
        ) }
    </>;
}

export const InternshipState = ({ internship }: { internship: InternshipSubmission }) => {
    const studentDataState = internship.intern && isStudentDataComplete(internship.intern) ? "accepted" : null;
    const proposalState = internship.state;
    const ippState = internship.ipp?.state || null;
    const reportState = internship.report?.state || null;
    const gradeState = internship.grade ? "accepted" : null;
    const approvalState = internship.approvals.reduce<SubmissionStatus | null>((status, document) => {
        switch (status) {
            case "awaiting":
                return status;
            case "declined":
                return document.state === "awaiting" ? document.state : status;
            case "draft":
                return ["awaiting", "declined"].includes(document.state) ? document.state : status;
            default:
                return document.state;
        }
    }, null);

    const { t } = useTranslation();
    const spacing = useSpacing(0.25);

    return <div className={ spacing.horizontal } style={{ display: "flex" }}>
        <StudentAction internship={ internship }>
            <StepState state={ studentDataState } label={ t("steps.personal-data.header") } icon={ <Account /> } />
        </StudentAction>
        <ProposalAction internship={ internship }>
            <StepState state={ proposalState } label={ t("steps.internship-proposal.header") } icon={ <BriefcaseAccount /> } />
        </ProposalAction>
        <IPPAction internship={ internship }>
            <StepState state={ ippState } label={ t("steps.plan.header") } icon={ <FormatPageBreak /> } />
        </IPPAction>
        <StepState state={ reportState } label={ t("steps.report.header") } icon={ <FileChartOutline /> } />
        <StepState state={ gradeState } label={ t("steps.grade.header") } icon={ <Star /> } />
        <StepState state={ approvalState } label={ t("steps.approvals.header") } icon={ <CertificateOutline/> }
                   style={ approvalState ? {} : { opacity: 0.2 } }
        />
    </div>
}

export const InternshipManagement = ({ edition }: EditionManagementProps) => {
    const { t } = useTranslation("management");
    const [result, setInternshipsPromise] = useAsyncState<InternshipSubmission[]>();
    const [selected, setSelected] = useState<InternshipSubmission[]>([]);
    const spacing = useSpacing(2);

    const updateInternshipList = () => {
        setInternshipsPromise(api.internship.all(edition));
    }

    useEffect(updateInternshipList, []);

    const GradeAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleGradeSubmission = async (grade: number) => {
            await api.internship.grade(internship as Internship, grade);
            setOpen(false);
            updateInternshipList();
        }

        return <>
            <Tooltip title={ t("internship.grade") as string }>
                <IconButton onClick={ () => setOpen(true) }><StickerCheckOutline/></IconButton>
            </Tooltip>
            { createPortal(
                <GradeDialog onSubmit={ handleGradeSubmission } internship={ internship } open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const columns: Column<InternshipSubmission>[] = [
        {
            title: t("internship.column.student"),
            render: internship => <>{internship.intern?.name} {internship.intern?.surname}</>,
        },
        {
            title: t("internship.column.album"),
            field: "intern.albumNumber",
        },
        {
            title: t("internship.column.type"),
            field: "type.label.pl",
        },
        {
            title: t("internship.column.status"),
            render: summary => <InternshipState internship={ summary } />
        },
        {
            title: t("internship.column.grade"),
            field: "grade",
            width: 0,
        },
        actionsColumn(internship => <>
            { canGrade(internship) && <GradeAction internship={ internship } /> }
        </>)
    ];

    return <Page>
        <Page.Header maxWidth="lg">
            <EditionManagement.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </EditionManagement.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <Button onClick={ updateInternshipList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>

            </BulkActions> }
            <Async async={ result } keepValue>{
                internships => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t(title) }/> }
                    columns={ columns }
                    data={ internships }
                    onSelectionChange={ internships => setSelected(internships) }
                    options={ { selection: true, pageSize: 10 } }
                />
            }</Async>
        </Container>
    </Page>
}
