import React, { useCallback, useContext } from "react";
import { useTranslation } from "react-i18next";
import { useRouteMatch, Link as RouterLink } from "react-router-dom";
import { Page } from "@/pages/base";
import { Container, Link, Paper, Typography } from "@material-ui/core";
import { Management, ManagementLink } from "@/management/main";
import { Edition } from "@/data/edition";
import {
    AccountMultiple,
    BriefcaseAccount,
    CertificateOutline,
    CogOutline,
    FileAccountOutline,
    FileChartOutline,
    FileQuestionOutline,
    FormatPageBreak
} from "mdi-material-ui";
import { route, routes, Routes } from "@/routing";
import { useSpacing } from "@/styles";
import { createStyles, makeStyles, Theme } from "@material-ui/core/styles";
import { useAsync } from "@/hooks";
import api from "@/management/api";
import { Async } from "@/components/async";
import { OneOrMany } from "@/helpers";

const useSectionStyles = makeStyles((theme: Theme) => createStyles({
    header: {
        padding: theme.spacing(2),
        paddingBottom: 0,
        fontWeight: 'bold',
        color: theme.palette.grey["800"],
    },
}))

export function title(edition: Edition) {
    return `${ edition.course.name } - ${ edition.startDate.year() }`
}

export const EditionContext = React.createContext<Edition | null>(null);

export const EditionManagement = ({ edition }: EditionManagementProps) => {
    const { t } = useTranslation("management");

    const spacing = useSpacing(2);
    const classes = useSectionStyles();

    return <Page>
        <Page.Header>
            <Management.Breadcrumbs>
                <Typography color="textPrimary">{ title(edition) }</Typography>
            </Management.Breadcrumbs>
            <Page.Title>{ title(edition) }</Page.Title>
        </Page.Header>
        <Container className={ spacing.vertical }>
            <Paper elevation={ 2 }>
                <Typography className={ classes.header }>{ t("edition.manage.internships") }</Typography>
                <Management.Menu>
                    <ManagementLink icon={ <BriefcaseAccount/> } route={ route("management:edition_internships", { edition: edition.id || "" }) }>
                        { t("management:edition.internships.title") }
                    </ManagementLink>
                    <ManagementLink icon={ <FileQuestionOutline/> } route={ route("management:edition_proposals", { edition: edition.id || "" }) }>
                        { t("management:edition.proposals.title") }
                    </ManagementLink>
                    <ManagementLink icon={ <FormatPageBreak/> } route={ route("management:edition_ipp_index", { edition: edition.id || "" }) }>
                        { t("management:edition.ipp.title") }
                    </ManagementLink>
                    <ManagementLink icon={ <FileChartOutline/> } route={ route("management:edition_reports", { edition: edition.id || "" }) }>
                        { t("management:edition.reports.title") }
                    </ManagementLink>
                    <ManagementLink icon={ <CertificateOutline/> } route={ route("management:edition_report_form", { edition: edition.id || "" }) }>
                        { t("management:edition.dean-approvals.title") }
                    </ManagementLink>
                </Management.Menu>
            </Paper>
            <Paper elevation={ 2 }>
                <Typography className={ classes.header }>{ t("edition.manage.management") }</Typography>
                <Management.Menu>
                    <ManagementLink icon={ <FormatPageBreak/> } route={ route("management:edition_schema", { edition: edition.id || "" }) }>
                        { t("management:edition.settings.schema") }
                    </ManagementLink>
                    <ManagementLink icon={ <CogOutline/> } route={ route("management:edition_settings", { edition: edition.id || "" }) }>
                        { t("management:edition.settings.title") }
                    </ManagementLink>
                </Management.Menu>
            </Paper>
        </Container>
    </Page>
}

EditionManagement.Breadcrumbs = ({ children }: { children: OneOrMany<React.ReactChild> }) => {
    const edition = useContext<Edition | null>(EditionContext);

    return <Management.Breadcrumbs>
        { edition && (children
            ? <Link to={ route("management:edition_manage", { edition: edition.id || "" }) } component={ RouterLink }>{ title(edition) }</Link>
            : <Typography color="textPrimary">{ title(edition) }</Typography>
        ) }
        { children }
    </Management.Breadcrumbs>
}

export type EditionManagementProps = {
    edition: Edition;
}

export const EditionRouter = () => {
    const { params } = useRouteMatch();

    const edition = useAsync<Edition>(useCallback(() => api.edition.details(params.edition), [params.edition]));

    return <Async async={ edition }>{
        result => <EditionContext.Provider value={ result }>
            <Routes routes={ routes.filter(route => (route.tags || []).includes("edition")) } edition={ result }/>
        </EditionContext.Provider>
    }</Async>
}
