import React, { useCallback, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useAsync, useAsyncState } from "@/hooks";
import { useSpacing } from "@/styles";
import api from "@/management/api";
import { Box, Button, Container, IconButton, Tooltip, Typography } from "@material-ui/core";
import MaterialTable, { Column } from "material-table";
import { actionsColumn } from "@/management/common/helpers";
import { FileFind, Refresh, StickerCheckOutline, StickerRemoveOutline } from "mdi-material-ui";
import { Page } from "@/pages/base";
import { Actions } from "@/components";
import { BulkActions } from "@/management/common/BulkActions";
import { Async } from "@/components/async";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { EditionManagement, EditionManagementProps } from "@/management/edition/manage";
import { AcceptSubmissionDialog, DiscardSubmissionDialog } from "@/components/acceptance-action";
import { ProposalPreview } from "@/components/proposalPreview";
import { InternshipSubmission } from "@/management/api/internship";
import { canAccept, canDiscard, StateLabel } from "@/management/edition/proposal/common";
import { createPortal } from "react-dom";
import { Internship } from "@/data";
import { InternshipDetailsDialog } from "@/management/edition/proposal/details";

const title = "edition.internships.title";

export const ProposalManagement = ({ edition }: EditionManagementProps) => {
    const { t } = useTranslation("management");
    const [result, setInternshipsPromise] = useAsyncState<InternshipSubmission[]>();
    const [selected, setSelected] = useState<InternshipSubmission[]>([]);
    const spacing = useSpacing(2);

    const updateInternshipList = () => {
        setInternshipsPromise(api.internship.all(edition));
    }

    useEffect(updateInternshipList, []);

    const handleSubmissionDiscard = async (internship: InternshipSubmission, comment: string) => {
        await api.internship.discard(internship as Internship, comment);
        updateInternshipList();
    }
    const handleSubmissionAccept = async (internship: InternshipSubmission, comment?: string) => {
        await api.internship.accept(internship as Internship, comment);
        updateInternshipList();
    }

    const AcceptAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleAccept = (comment?: string) => {
            setOpen(false);
            handleSubmissionAccept(internship, comment);
        }

        return <>
            <Tooltip title={ t("translation:accept") as any }><IconButton onClick={ () => setOpen(true) }><StickerCheckOutline /></IconButton></Tooltip>
            { createPortal(
                <AcceptSubmissionDialog onAccept={ handleAccept } label="internship" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const DiscardAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleDiscard = (comment: string) => {
            setOpen(false);
            handleSubmissionDiscard(internship, comment);
        }

        return <>
            <Tooltip title={ t("translation:discard") as any }><IconButton onClick={ () => setOpen(true) }><StickerRemoveOutline /></IconButton></Tooltip>
            { createPortal(
                <DiscardSubmissionDialog onDiscard={ handleDiscard } label="internship" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const PreviewAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleDiscard = (comment: string) => {
            setOpen(false);
            handleSubmissionDiscard(internship, comment);
        }

        const handleAccept = (comment?: string) => {
            setOpen(false);
            handleSubmissionAccept(internship, comment);
        }

        return <>
            <Tooltip title={ t("translation:preview") as any }><IconButton onClick={ () => setOpen(true) }><FileFind /></IconButton></Tooltip>
            { createPortal(
                <InternshipDetailsDialog onDiscard={ handleDiscard } onAccept={ handleAccept } internship={ internship } open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const InternshipDetails = ({ summary }: { summary: InternshipSubmission }) => {
        const internship = useAsync(useCallback(() => api.internship.get(summary.id || ""), [ summary.id ]))
        return <Box m={ 3 }><Async async={ internship }>{ internship => <ProposalPreview proposal={ internship as Internship } /> }</Async> </Box>
    }

    const columns: Column<InternshipSubmission>[] = [
        {
            title: t("internship.column.student"),
            render: internship => <>{internship.intern?.name} {internship.intern?.surname}</>,
        },
        {
            title: t("internship.column.album"),
            field: "intern.albumNumber",
        },
        {
            title: t("internship.column.type"),
            field: "type.label.pl",
        },
        {
            title: t("internship.column.changed"),
            render: summary => summary.changed?.format("yyyy-MM-DD")
        },
        {
            title: t("internship.column.status"),
            render: summary => <StateLabel state={ summary.state } />
        },
        actionsColumn(internship => <>
            { canAccept(internship) && <AcceptAction internship={ internship } /> }
            { canDiscard(internship) && <DiscardAction internship={ internship } /> }
            <PreviewAction internship={ internship } />
        </>)
    ];

    return <Page>
        <Page.Header maxWidth="lg">
            <EditionManagement.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </EditionManagement.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <Button onClick={ updateInternshipList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>

            </BulkActions> }
            <Async async={ result } keepValue>{
                internships => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t(title) }/> }
                    columns={ columns }
                    data={ internships }
                    onSelectionChange={ internships => setSelected(internships) }
                    options={ { selection: true, pageSize: 10 } }
                    detailPanel={ summary => <InternshipDetails summary={ summary } /> }
                />
            }</Async>
        </Container>
    </Page>
}
