import React, { useCallback, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useAsync, useAsyncState } from "@/hooks";
import { useSpacing } from "@/styles";
import api from "@/management/api";
import { Box, Button, Container, IconButton, Typography } from "@material-ui/core";
import MaterialTable, { Column } from "material-table";
import { actionsColumn } from "@/management/common/helpers";
import { FileDownloadOutline, FileFind, Refresh, StickerCheckOutline, StickerRemoveOutline } from "mdi-material-ui";
import { Page } from "@/pages/base";
import { Actions } from "@/components";
import { BulkActions } from "@/management/common/BulkActions";
import { Async } from "@/components/async";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { EditionManagement, EditionManagementProps } from "@/management/edition/manage";
import { AcceptSubmissionDialog, DiscardSubmissionDialog } from "@/components/acceptance-action";
import { InternshipSubmission } from "@/management/api/internship";
import { StateLabel } from "@/management/edition/proposal/common";
import { createPortal } from "react-dom";
import { Stateful } from "@/data";
import { Report } from "@/data/report";

const title = "edition.reports.title";

export const canAccept = (subject: Stateful | null) => !!(subject && ["declined", "awaiting"].includes(subject.state));
export const canDiscard = (subject: Stateful | null) => !!(subject && ["accepted", "awaiting"].includes(subject.state));

export const ReportManagement = ({ edition }: EditionManagementProps) => {
    const { t } = useTranslation("management");
    const [result, setInternshipsPromise] = useAsyncState<InternshipSubmission[]>();
    const [selected, setSelected] = useState<InternshipSubmission[]>([]);
    const spacing = useSpacing(2);

    const updateInternshipList = () => {
        setInternshipsPromise(api.internship.all(edition));
    }

    useEffect(updateInternshipList, []);

    const AcceptAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleSubmissionAccept = async (comment?: string) => {
            setOpen(false);
            await api.report.accept(internship.report as Report, comment);
            updateInternshipList();
        }

        return <>
            <IconButton onClick={ () => setOpen(true) }><StickerCheckOutline /></IconButton>
            { createPortal(
                <AcceptSubmissionDialog onAccept={ handleSubmissionAccept } label="plan" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const DiscardAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        const handleSubmissionDiscard = async (comment: string) => {
            setOpen(false);
            await api.report.discard(internship.report as Report, comment);
            updateInternshipList();
        }

        return <>
            <IconButton onClick={ () => setOpen(true) }><StickerRemoveOutline /></IconButton>
            { createPortal(
                <DiscardSubmissionDialog onDiscard={ handleSubmissionDiscard } label="plan" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const InternshipDetails = ({ summary }: { summary: InternshipSubmission }) => {
        return <Box m={ 3 }>
            { summary.report && JSON.stringify(summary.report) }
        </Box>
    }

    const columns: Column<InternshipSubmission>[] = [
        {
            title: t("internship.column.student"),
            render: internship => <>{internship.intern?.name} {internship.intern?.surname}</>,
        },
        {
            title: t("internship.column.album"),
            field: "intern.albumNumber",
        },
        {
            title: t("internship.column.type"),
            field: "type.label.pl",
        },
        {
            title: t("internship.column.changed"),
            render: summary => summary.changed?.format("yyyy-MM-DD")
        },
        {
            title: t("internship.column.status"),
            render: summary => <StateLabel state={ summary.report?.state || null } />
        },
        actionsColumn(internship => <>
            { canAccept(internship.report) && <AcceptAction internship={ internship } /> }
            { canDiscard(internship.report) && <DiscardAction internship={ internship } /> }
        </>)
    ];

    return <Page>
        <Page.Header maxWidth="lg">
            <EditionManagement.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </EditionManagement.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <Button onClick={ updateInternshipList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>

            </BulkActions> }
            <Async async={ result } keepValue>{
                internships => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t(title) }/> }
                    columns={ columns }
                    data={ internships }
                    onSelectionChange={ internships => setSelected(internships) }
                    options={ { selection: true, pageSize: 10 } }
                    detailPanel={ summary => <InternshipDetails summary={ summary } /> }
                />
            }</Async>
        </Container>
    </Page>
}
