import React, { useCallback, useEffect, useState } from "react";
import { Page } from "@/pages/base";
import { Management } from "@/management/main";
import { Box, Button, Container, IconButton, Tooltip, Typography } from "@material-ui/core";
import { useTranslation } from "react-i18next";
import MaterialTable, { Column } from "material-table";
import { actionsColumn, fieldComparator, multilingualStringComparator } from "@/management/common/helpers";
import { MultilingualCell } from "@/management/common/MultilangualCell";
import { ReportFieldDefinition } from "@/data/report";
import { Formik } from "formik";
import { CustomField } from "@/forms/report";
import { Add, Edit } from "@material-ui/icons";
import { createPortal } from "react-dom";
import { createDeleteAction } from "@/management/common/DeleteResourceAction";
import { EditFieldDefinitionDialog } from "@/management/report/fields/edit";
import api from "@/management/api";
import { useAsync, useAsyncState } from "@/hooks";
import { Async } from "@/components/async";
import { Actions } from "@/components";
import { Refresh } from "mdi-material-ui";
import { useSpacing } from "@/styles";

const title = "report-fields.title";

export const FieldPreview = ({ field }: { field: ReportFieldDefinition }) => {
    return <Formik initialValues={{}} onSubmit={() => {}}>
        <CustomField field={ field }/>
    </Formik>
}

export const ReportFields = () => {
    const { t } = useTranslation("management");
    const [fields, setFieldsPromise] = useAsyncState<ReportFieldDefinition[]>();

    const updateFieldList = () => {
        setFieldsPromise(api.field.all());
    }

    useEffect(updateFieldList, []);

    const handleFieldDeletion = () => {}

    const CreateFieldAction = () => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handleFieldCreation = async (value: ReportFieldDefinition) => {
            await api.field.save(value);
            setOpen(false);
        }

        return <>
            <Button variant="contained" color="primary" startIcon={ <Add /> } onClick={ () => setOpen(true) }>{ t("create") }</Button>
            { open && createPortal(
                <EditFieldDefinitionDialog open={ open } onSave={ handleFieldCreation } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const DeleteFieldAction  = createDeleteAction<ReportFieldDefinition>({ label: field => field.label.pl, onDelete: handleFieldDeletion })
    const EditFieldAction = ({ field }: { field: ReportFieldDefinition }) => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handleFieldSave = async (field: ReportFieldDefinition) => {
            await api.field.save(field);
            setOpen(false);
        }

        return <>
            <Tooltip title={ t("actions.edit") as any }>
                <IconButton onClick={ () => setOpen(true) }><Edit /></IconButton>
            </Tooltip>
            { open && createPortal(
                <EditFieldDefinitionDialog open={ open } onSave={ handleFieldSave } field={ field } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const columns: Column<ReportFieldDefinition>[] = [
        {
            title: t("report-field.field.label"),
            customSort: fieldComparator('label', multilingualStringComparator),
            cellStyle: { whiteSpace: "nowrap" },
            render: field => <MultilingualCell value={ field.label }/>,
        },
        {
            title: t("report-field.field.type"),
            cellStyle: { whiteSpace: "nowrap" },
            render: field => t(`report-field.type.${field.type}`),
        },
        actionsColumn(field => <>
            <EditFieldAction field={ field }/>
            <DeleteFieldAction resource={ field }/>
        </>),
    ]

    const spacing = useSpacing(2);

    return <Page>
        <Page.Header maxWidth="lg">
            <Management.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </Management.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <CreateFieldAction />
                <Button onClick={ updateFieldList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            <Async async={ fields }>
                { fields => <MaterialTable
                    columns={ columns }
                    data={ fields }
                    title={ t(title) }
                    detailPanel={ field => <Box p={3}><FieldPreview field={ field } /></Box> }
                /> }
            </Async>
        </Container>
    </Page>
}
